@@ -124,10 +124,10 @@ passed separately. If we're making anything fancier than a simple 2D or 3D plot
 (2- and 3- tuples respectively) then the 'tuplesize' curve option MUST be passed
 in.
 
-Furthermore, broadcasting is fully supported, so multiple curves can be plotted
-by stacking data inside the passed-in arrays. Broadcasting works across curve
-options also, so things like curve labels and styles can also be stacked inside
-arrays. An example:
+Furthermore, broadcasting (https://docs.scipy.org/doc/numpy/user/basics.broadcasting.html)
+is fully supported, so multiple curves can be plotted by stacking data inside
+the passed-in arrays. Broadcasting works across curve options also, so things
+like curve labels and styles can also be stacked inside arrays. An example:
 
     th    = np.linspace(0, 6*np.pi, 200)
     z     = np.linspace(0, 5,       200)
@@ -806,7 +806,7 @@ import numpy as np
 import numpysane as nps
 
 # setup.py assumes the version is a simple string in '' quotes
-__version__ = '0.27'
+__version__ = '0.29'
 
 # note that 'with' is both a known plot and curve option
 knownPlotOptions = frozenset(('dump', 'ascii', 'log', 'notest', 'wait',
@@ -951,12 +951,27 @@ class gnuplotlib:
         except:
             self.fdDupSTDOUT = None
 
+
+        # I need this to make fdDupSTDOUT available to the child gnuplot. This
+        # would happen by default, but in python3 I need to do this extra thing
+        # for some reason. And it's a new thing that didn't exist in python2, so
+        # I need to explicitly allow this to fail in python2
+        try:
+            os.set_inheritable(self.fdDupSTDOUT, True)
+        except AttributeError:
+            pass
+
         self.gnuplotProcess = \
             subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
 
                              # required to "autoflush" writes
                              bufsize=0,
 
+                             # I need this to make fdDupSTDOUT available to the
+                             # child gnuplot. close_fds=False was default in
+                             # python2, but was changed in python3
+                             close_fds = False,
+
                              # This was helpful in python3 to implicitly
                              # encode() strings, but it broke the
                              # select()/read() mechanism: select() would
@@ -1299,7 +1314,12 @@ defaults are acceptable, use 'hardcopy' only, otherwise use 'terminal' and
         # yet arrived. I thus print out a checkpoint message and keep reading the
         # child's STDERR pipe until I get that message back. Any errors would have
         # been printed before this
-        if self.plotOptions.get('notest') and flags != 'waitforever':
+        waitforever                = re.search('waitforever',                flags)
+        final                      = re.search('final',                      flags)
+        printwarnings              = re.search('printwarnings',              flags)
+        ignore_known_test_failures = re.search('ignore_known_test_failures', flags)
+
+        if self.plotOptions.get('notest') and not waitforever and not final and not printwarnings:
             return None, None
 
         checkpoint = "gpsync{}xxx".format(self.sync_count)
@@ -1322,7 +1342,7 @@ defaults are acceptable, use 'hardcopy' only, otherwise use 'terminal' and
             self._logEvent("Trying to read from gnuplot")
 
             rlist,wlist,xlist = select.select([self.gnuplotProcess.stderr],[], [],
-                                              None if flags == 'waitforever' else 15)
+                                              None if waitforever else 15)
 
             if rlist:
                 # read a byte. I'd like to read "as many bytes as are
@@ -1351,7 +1371,7 @@ and/or gnuplot itself. Please report this as a gnuplotlib bug''')
         warningre = re.compile(r'^.*(?:warning:\s*(.*?)\s*$)\n?', re.M + re.I)
         warnings  = warningre.findall(fromerr)
 
-        if flags == 'printwarnings':
+        if printwarnings:
             for w in warnings:
                 sys.stderr.write("Gnuplot warning: {}\n".format(w))
 
@@ -1372,7 +1392,7 @@ and/or gnuplot itself. Please report this as a gnuplotlib bug''')
         #    so any out-of-boundedness of the test data is irrelevant
         #
         # 3. image grid complaints
-        if flags == 'ignore_known_test_failures':
+        if ignore_known_test_failures:
 
             r = re.compile(r'''^gnuplot>\s*(?:{}|e\b).*$                  # report of the actual invalid command
                                \n^\s+\^\s*$                               # ^ mark pointing to where the error happened
@@ -1499,6 +1519,8 @@ labels with spaces in them
         else:
             nps.mv(nps.cat(*curve['_data']), 0, -1).astype(np.float64,copy=False).tofile(pipe)
 
+        self._logEvent("Sent the data to child process")
+
 
     def _getPlotCmd(self, curves):
 
@@ -1945,6 +1967,25 @@ labels with spaces in them
         for curve in curves:
             self._sendCurve(curve)
 
+        # There's some bug in gnuplot right now, where it sometimes reads too
+        # many bytes after receiving inline data, which swallows the initial
+        # bytes in a subsequent command, breaking things. I workaround by
+        # stuffing newlines into the pipe. These don't do anything, and gnuplot
+        # is allowed to steal some number of them without breaking anything. I
+        # running gnuplot=5.2.6+dfsg1-1 on Debian. I can tickle the bug by doing
+        # this:
+        #   gp.plot(np.arange(5))
+        # Error:
+        # ...
+        #   File "/home/dima/projects/gnuplotlib/gnuplotlib.py", line 1221, in _safelyWriteToPipe
+        #     raise GnuplotlibError(barfmsg)
+        # gnuplotlib.GnuplotlibError: Gnuplot error: '
+        # "
+        #          ^
+        #          line 0: invalid command
+        # ' while sending cmd 'set output'
+        self._printGnuplotPipe('\n\n\n\n')
+
         if self.plotOptions.get('terminal') == 'gp':
             self._printGnuplotPipe('pause mouse close\n')
             self._dumpPipe.close()
@@ -1961,6 +2002,11 @@ labels with spaces in them
             if self.plotOptions.get('wait'):
                 self.wait()
 
+        # I force gnuplot to tell me it's done before exiting. Without this 'set
+        # terminal dumb' plots don't end up rendering anything: we kill the
+        # process before it has time to do anything
+        self._checkpoint('final printwarnings')
+
 
 
 
